@@ -2,10 +2,10 @@ package CGI::Lingua;
 
 use warnings;
 use strict;
-use Class::Autouse qw{Carp Locale::Language Locale::Object::Country Locale::Object::DB I18N::AcceptLanguage I18N::LangTags::Detect};
+use Class::Autouse qw{Carp Locale::Language Locale::Object::Country Locale::Object::DB I18N::AcceptLanguage I18N::LangTags::Detect Storable};
 
 use vars qw($VERSION);
-our $VERSION = '0.52';
+our $VERSION = '0.53';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ CGI::Lingua - Create a multilingual web page
 
 =head1 VERSION
 
-Version 0.52
+Version 0.53
 
 =cut
 
@@ -111,9 +111,35 @@ sub new {
 		Carp::croak('You must give a list of supported languages');
 	}
 
+	my $cache = $params{cache};
+	my $logger = $params{logger};
+	if($cache && $ENV{'REMOTE_ADDR'}) {
+		my $key = "$ENV{REMOTE_ADDR}/";
+		if($ENV{'HTTP_ACCEPT_LANGUAGE'}) {
+			$key .= "$ENV{HTTP_ACCEPT_LANGUAGE}/";
+		}
+		$key .= join('/', @{$params{supported}});
+		if($logger) {
+			$logger->debug("Looking in cache for $key");
+		}
+		my $rc = $cache->get($key);
+		if($rc) {
+			if($logger) {
+				$logger->debug('Found - thawing');
+			}
+			$rc = Storable::thaw($rc);
+			$rc->{_logger} = $logger;
+			$rc->{_syslog} = $params{syslog};
+			$rc->{_cache} = $cache;
+			$rc->{_supported} = $params{supported};
+
+			return $rc;
+		}
+	}
+
 	return bless {
 		_supported => $params{supported}, # List of languages (two letters) that the application
-		_cache => $params{cache},	# CHI
+		_cache => $cache,	# CHI
 		# _rlanguage => undef,	# Requested language
 		# _slanguage => undef,	# Language that the website should display
 		# _sublanguage => undef,	# E.g. United States for en-US if you want American English
@@ -123,13 +149,57 @@ sub new {
 		# _locale => undef,	# Locale::Object::Country
 		_syslog => $params{syslog},
 		_dont_use_ip => $params{dont_use_ip} || 0,
-		_logger => $params{logger},
+		_logger => $logger,
 		_have_ipcountry => -1,	# -1 = don't know
 		_have_geoip => -1,	# -1 = don't know
 		_have_geoipfree => -1,	# -1 = don't know
 	}, $class;
 }
 
+sub DESTROY {
+	if(defined($^V) && ($^V ge 'v5.14.0')) {
+		return if ${^GLOBAL_PHASE} eq 'DESTRUCT';	# >= 5.14.0 only
+	}
+	unless($ENV{'REMOTE_ADDR'}) {
+		return;
+	}
+	my $self = shift;
+	return unless(ref($self));
+
+	my $cache = $self->{_cache};
+	return unless($cache);
+
+	my $logger = $self->{_logger};
+
+	my $key = "$ENV{REMOTE_ADDR}/";
+	if($ENV{'HTTP_ACCEPT_LANGUAGE'}) {
+		$key .= "$ENV{HTTP_ACCEPT_LANGUAGE}/";
+	}
+	$key .= join('/', @{$self->{_supported}});
+	return if($cache->get($key));
+	if($logger) {
+		$logger->trace("Storing self in cache as $key");
+	}
+
+	my $copy = bless {
+		_slanguage => $self->{_slanguage},
+		_slanguage_code_alpha2 => $self->{_slanguage_code_alpha2},
+		_country => $self->{_country},
+		_rlanguage => $self->{_rlanguage},
+		_dont_use_ip => $self->{_dont_use_ip},
+		_have_ipcountry => $self->{_have_ipcountry},
+		_have_geoip => $self->{_have_geoip},
+		_have_geoipfree => $self->{_have_geoipfree}
+	}, ref($self);
+
+	# All of these crash, presumably something recursive is going on
+	# my $copy = Clone::clone($self);
+	# my $storable = Storable::nfreeze(Storable::dclone($self));
+	# my $storable = Storable::dclone($self);
+
+	$cache->set($key, Storable::nfreeze($copy), '1 month');
+}
+
 # Emit a warning message somewhere
 sub _warn {
 	my ($self, $params) = @_;
@@ -973,7 +1043,14 @@ sub _code2country
 			$self->{_logger}->trace("_code2country $code");
 		}
 	}
-	return Locale::Object::Country->new(code_alpha2 => $code);
+	local $SIG{__WARN__} = sub {
+		if($_[0] !~ /No result found in country table/) {
+			warn $_[0];
+		}
+	};
+	my $rc = Locale::Object::Country->new(code_alpha2 => $code);
+	local $SIG{__WARN__} = 'DEFAULT';
+	return $rc;
 }
 
 # Wrapper to Locale::Object::Country->name which makes use of the cache
